/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2015   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  FILE: lbpplatontest.cc
 *
 */


#include "image3d.h"
#include "descriptors/lbpplaton.h"

using namespace std;
using namespace i3d;


int main(int argc, char* argv[])
{
	if ((argc != 3) || (atoi(argv[2]) < 1) || (atoi(argv[2]) > 4)) {
		cerr << argv[0] << " ... evaluation of LBP-platonic features \n\n"
		"Copyright (C) 2015 Centre for Biomedical Image Analysis \n"
		"This is free software; see the source for copying conditions. "
		"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
		"A PARTICULAR PURPOSE.\n\n";
		cerr << "SYNTAX" << endl << "\t" << "lbpplatontest.exe input_image <option> " << endl << endl;
		cerr << "OPTION" << endl <<
		"\t <int> ... could be 1..4, where" << endl <<
		"\t 1 = tetrahedron \n\t 2 = hexahedron \n\t 3 = octahedron \n\t 4 = icosahedron \n" << endl;
		cerr << "EXAMPLE" << endl <<
		"lbpplatontest.exe centromere-028-test-16-37-p.png 1" << endl << endl;
		exit(1);
	}

	try  
	{
		ImgVoxelType vt = ReadImageType(argv[1]);
		vector<double> final_vector;
		switch (vt)
		{
			case Gray8Voxel: 
			{
				Image3d<GRAY8> img(argv[1]);
				if (img.GetSizeZ() > 1) 
					ComputeLBP_PLATON(img, (GRAY8) atoi(argv[2]), final_vector);
				else 
				{
					cerr << "LBP-platonic are defined only for 3D images. For 2D images use lbptest.exe" << endl;
					exit(1);
				}				
				break;
			}
			case Gray16Voxel: 
			{
				Image3d<GRAY16> img(argv[1]);
				if (img.GetSizeZ() > 1) 
					ComputeLBP_PLATON(img, (GRAY16) atoi(argv[2]), final_vector);
				else 
				{
					cerr << "LBP-platonic are defined only for 3D images. For 2D images use lbptest.exe" << endl;
					exit(1);
				}
				break;
			}
			default: cerr << "wrong image type" << endl;
		}

		// print the feature vector
		for (unsigned i = 0; i < final_vector.size(); i++)
		{
			if (i > 0) cout << ",";		
			cout << final_vector[i]; 
		}
		cout << endl; 
	}
	catch (IOException& e) {
		cerr << e <<endl;
		exit(1);
	}
	catch (InternalException& e) {
		cerr << e <<endl;
		exit(1);
	}
	catch (bad_alloc&) {
		cerr << "Not enough memory." << endl;
		exit(1);
	}
	catch (...) {
		cerr << "lbpplatontest: Unknown exception." << endl;
		exit(1);
	}

  return 0;
}