#include "image3d.h"
#include "imgfiles.h"
#include <stdio.h>
#include <iostream>

using namespace std;
using namespace i3d;

void usage(const char *str)
{
   cerr << str << " ... simple image checker \n" 
	"Copyright (C) 2023 Centre for Biomedical Image Analysis \n"
	"This is free software; see the source for copying conditions. "
	"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
	"A PARTICULAR PURPOSE.\n\n";
	 cerr << "Usage:" << endl << endl;
	 cerr << "$> " << str << " <input file> " << endl << endl;
	 cerr << endl;
}

int main (int argc, char *argv[])
{
	 if (argc < 2)
	 {
		  usage(argv[0]);
		  exit(-1);
	 }
  try
  {
    ImageHeader ih = ReadImageHeader(argv[1]);
	 int dim;

    if (ih.size.z == 1)
		dim = 2;
	 else
      dim = 3;

	 switch (ih.type)
      {
	  case BinaryVoxel:
				 {
					  cout << "binary" << " "  << dim << endl;
					  break;
				 }
	  case Gray8Voxel:
				 {
					  cout << "grayscale" << " "  << dim << endl;
					  break;
				 }
	  case Gray16Voxel:
				 {
					  cout << "grayscale" << " "  << dim << endl;
					  break;
				 }
	  case RGBVoxel:
				 {
					  cout << "rgb (3x8bit)" << " "  << dim << endl;
					  break;
				 }
	  case RGB16Voxel:
				 {
					  cout << "rgb (3x16bit)" << " "  << dim << endl;
					  break;
				 }
	  case FloatVoxel:
				 {
					  cout << "float" << " "  << dim << endl;
					  break;
				 }
      default:
				 {
					  cerr << argv[0] << ": Unsupported data type." << endl;
					  exit(-1);
				 }
      }
  }
  catch (IOException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (InternalException & e)
  {
    cerr << e << endl;
    exit (-1);
  }
  catch (bad_alloc &)
  {
    cerr << "Not enough memory." << endl;
    exit (-1);
  }
  catch (...)
  {
    cerr << "Unknown exception." << endl;
    exit (-1);
  }

  return 0;
}
