#include <stdio.h>
#include <stdlib.h>
#include "image3d.h"
#include "threshold.h"

//---------------------------------------------

using namespace std;
using namespace i3d;

int main(int argc, char *argv[])
{
	Image3d<bool> out;

	if (argc != 5) 
	{
   	cerr << argv[0] << " ... thresholding of grayscale/float images \n\n"
			"Copyright (C) 2023 Centre for Biomedical Image Analysis \n"
		"This is free software; see the source for copying conditions. "
		"There is NO\nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
		"A PARTICULAR PURPOSE.\n\n";
	    cerr << "Usage: "<< 
				argv[0] << " <input image> <output image> <lower threshold> <upper threshold>" << endl;
	 cerr << " - input image: the grayscale image to be thresholded" << endl;
    cerr << " - output image: the binary mask that should be the output of the threshold" << endl;
	 cerr << " - lower threshold: number" << endl <<
				" - uppper threshold: number \n" << endl;
   exit(1);
	} 

  try  
  {
		ImgVoxelType vt = ReadImageType(argv[1]);

		switch (vt)
		{
			 case Gray8Voxel: 
						{
							 Image3d<GRAY8> img(argv[1]);
							 Threshold(img, out, (GRAY8) atoi(argv[3]), (GRAY8) atoi(argv[4]));
							 out.SaveImage(argv[2]);
							 break;
						}
			 case Gray16Voxel: 
						{
							 Image3d<GRAY16> img(argv[1]);
							 Threshold(img, out, (GRAY16) atoi(argv[3]), (GRAY16) atoi(argv[4]));
							 out.SaveImage(argv[2]);
							 break;
						}
			 case FloatVoxel: 
						{
							 Image3d<float> img(argv[1]);
							 Threshold(img, out, (float) atof(argv[3]), (float) atof(argv[4]));
							 out.SaveImage(argv[2]);
							 break;
						}
			 default: cerr << "Wrong input image type" << endl;
		}
  }
  catch (IOException& e) {
    cerr << e <<endl;
    exit(1);
  }
  catch (InternalException& e) {
    cerr << e <<endl;
    exit(1);
  }
  catch (bad_alloc&) {
    cerr << "Not enough memory." << endl;
    exit(1);
  }
  catch (...) {
    cerr << "threshold: Unknown exception." << endl;
    exit(1);
  }

  return 0;
	
}

