/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2006   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * neighbour_indices.cc
 *
 * Neighbour indices intended for various computational grids
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006 - 2011
 */

#include "neighbour_indices.h"

namespace i3d
{
	namespace fls
	{
		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                  Class NeighbourIndices2D_4                 //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void NeighbourIndices2D_4::ApplyShifts(Type::Size i)
		{ 
			m_buffer[0] = i - 1;
			m_buffer[1] = i + 1;
			m_buffer[2] = i - m_grid.m_sx;
			m_buffer[3] = i + m_grid.m_sx;
		}

		/**********************************************************************/

		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                  Class NeighbourIndices2D_8                 //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void NeighbourIndices2D_8::ApplyShifts(Type::Size i)
		{ 
			m_buffer[0] = i - 1;
			m_buffer[1] = i + 1;
			m_buffer[2] = i - m_grid.m_sx;
			m_buffer[3] = i + m_grid.m_sx;
			m_buffer[4] = i - m_grid.m_sx - 1;
			m_buffer[5] = i - m_grid.m_sx + 1;
			m_buffer[6] = i + m_grid.m_sx - 1;
			m_buffer[7] = i + m_grid.m_sx + 1;
		}

		/**********************************************************************/

		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                  Class NeighbourIndices3D_6                 //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void NeighbourIndices3D_6::ApplyShifts(Type::Size i)
		{ 
			m_buffer[0] = i - 1;
			m_buffer[1] = i + 1;
			m_buffer[2] = i - m_grid.m_sx;
			m_buffer[3] = i + m_grid.m_sx;
			m_buffer[4] = i - m_grid.m_slice;
			m_buffer[5] = i + m_grid.m_slice;
		}

		/**********************************************************************/

		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                  Class NeighbourIndices3D_18                //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void NeighbourIndices3D_18::ApplyShifts(Type::Size i)
		{ 
			m_buffer[0] = i - 1;
			m_buffer[1] = i + 1;
			m_buffer[2] = i - m_grid.m_sx;
			m_buffer[3] = i + m_grid.m_sx;
			m_buffer[4] = i - m_grid.m_slice;
			m_buffer[5] = i + m_grid.m_slice;
			m_buffer[6] = i - m_grid.m_sx - 1;
			m_buffer[7] = i - m_grid.m_sx + 1;
			m_buffer[8] = i + m_grid.m_sx - 1;
			m_buffer[9] = i + m_grid.m_sx + 1;
			m_buffer[10] = i - m_grid.m_slice - 1;
			m_buffer[11] = i - m_grid.m_slice + 1;
			m_buffer[12] = i - m_grid.m_slice - m_grid.m_sx;
			m_buffer[13] = i - m_grid.m_slice + m_grid.m_sx;
			m_buffer[14] = i + m_grid.m_slice - 1;
			m_buffer[15] = i + m_grid.m_slice + 1;
			m_buffer[16] = i + m_grid.m_slice - m_grid.m_sx;
			m_buffer[17] = i + m_grid.m_slice + m_grid.m_sx;
		}

		/**********************************************************************/

		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                  Class NeighbourIndices3D_26                //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		void NeighbourIndices3D_26::ApplyShifts(Type::Size i)
		{
			m_buffer[0] = i - 1;
			m_buffer[1] = i + 1;
			m_buffer[2] = i - m_grid.m_sx;
			m_buffer[3] = i + m_grid.m_sx;
			m_buffer[4] = i - m_grid.m_slice;
			m_buffer[5] = i + m_grid.m_slice;
			m_buffer[6] = i - m_grid.m_sx - 1;
			m_buffer[7] = i - m_grid.m_sx + 1;
			m_buffer[8] = i + m_grid.m_sx - 1;
			m_buffer[9] = i + m_grid.m_sx + 1;
			m_buffer[10] = i - m_grid.m_slice - 1;
			m_buffer[11] = i - m_grid.m_slice + 1;
			m_buffer[12] = i - m_grid.m_slice - m_grid.m_sx;
			m_buffer[13] = i - m_grid.m_slice + m_grid.m_sx;
			m_buffer[14] = i + m_grid.m_slice - 1;
			m_buffer[15] = i + m_grid.m_slice + 1;
			m_buffer[16] = i + m_grid.m_slice - m_grid.m_sx;
			m_buffer[17] = i + m_grid.m_slice + m_grid.m_sx;
			m_buffer[18] = i - m_grid.m_slice - m_grid.m_sx - 1;
			m_buffer[19] = i - m_grid.m_slice - m_grid.m_sx + 1;
			m_buffer[20] = i - m_grid.m_slice + m_grid.m_sx - 1;
			m_buffer[21] = i - m_grid.m_slice + m_grid.m_sx + 1;
			m_buffer[22] = i + m_grid.m_slice - m_grid.m_sx - 1;
			m_buffer[23] = i + m_grid.m_slice - m_grid.m_sx + 1;
			m_buffer[24] = i + m_grid.m_slice + m_grid.m_sx - 1;
			m_buffer[25] = i + m_grid.m_slice + m_grid.m_sx + 1;
		}
		
		/**********************************************************************/

	} // namespace fls 

} // namespace i3d

