/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2006   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * object_manager.cc
 *
 * Manager responsible for efficient computation of average foreground and background object statistics
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006 - 2011
 */

#include "object_manager.h"

namespace i3d
{
	namespace fls
	{
		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                     Class ObjectManager                     //
		//                                                             //
		/////////////////////////////////////////////////////////////////
		
		/**********************************************************************/
		
		Type::Prec ObjectManager::VarForeground(Type::Size i) const 
		{ 
			Type::Prec var = m_obj_list.Ref(Label(i)).Variance(); 

			if (var == Type::Prec(0)) 
			{
				return var;
			}
			
			return std::log(var) + SqrDevForeground(i) / var;
		}

		/**********************************************************************/

		Type::Prec ObjectManager::VarBackground(Type::Size i) const 
		{ 
			Type::Prec var = m_obj_list.Ref(0).Variance(); 
			
			if (var == Type::Prec(0))
			{
				return var;
			}
			
			return std::log(var) + SqrDevBackground(i) / var; 
		}

		/**********************************************************************/

		void ObjectManager::Synchronize()
		{ 
			m_obj_list.Synchronize(); 
			--m_remain_iter;

			if (m_remain_iter == 0)
			{
				m_diff = m_obj_list.FlushDiff();
				m_remain_iter = m_num_iter;
			}
		}

		/**********************************************************************/

	} // namespace fls

} // namespace i3d

