/*
 * i3dlib - image manipulation library
 *
 * Copyright (C) 2000-2006   Centre for Biomedical Image Analysis (CBIA)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * routine.cc
 *
 * Computational routine interface
 *
 * Martin Maka (xmaska@fi.muni.cz) 2006 - 2011
 */

#include <iostream>
#include "routine.h"

namespace i3d
{
	namespace fls
	{
		/////////////////////////////////////////////////////////////////
		//                                                             //
		//                       Class Routine                         //
		//                                                             //
		/////////////////////////////////////////////////////////////////

		/**********************************************************************/

		template <class T> void Routine<T>::Iterate(Type::Size num_iter)
		{
			while (Ready() && num_iter > 0)
			{
				PerformIteration();
				--num_iter;
				++m_num_iter;
			}
		}

		/**********************************************************************/

		template <class T> void Routine<T>::IterateUntilConvergence(Type::Size max_num_iter)
		{
			while (Ready() && !Converged() && max_num_iter > 0)
			{
				PerformIteration();
				--max_num_iter;
				++m_num_iter;
			}
		}

		/**********************************************************************/

		template <class T> void Routine<T>::Info() const
		{
			std::cout << "=====================================================" << std::endl;
			std::cout << "Total number of iterations: " << m_num_iter << std::endl;
			std::cout << "Next iteration: " << (Ready() ? "Yes" : "No") << std::endl;
			std::cout << "Convergence: " << (Converged() ? "Yes" : "No") << std::endl;
		}

		/**********************************************************************/

		template class Routine<int>;

		/**********************************************************************/

	} // namespace fls

} // namespace i3d

