/*
 * thinning.cpp
 *
 * Function for thinning of binary images
 *
 * Marek Vinkler (xvinkl@mail.muni.cz) 2010
 * 
 */

#include "thinning.h"

typedef unsigned char uchar;

#define WH(conf, pos) (!(conf&(1<<pos)))
#define BL(conf, pos) ((conf&(1<<pos)))
#define WH_OR(conf, pos1, pos2) ((conf&(1<<pos1)) || (conf&(1<<pos2)))

namespace i3d
{
	void CreateLUT(std::vector<bool> &table)
	{
		// 2^25 bits
		size_t bits = 1<<25;
		table.resize(bits);

		// Fill the LUT
		for(size_t conf = 0; conf < bits; conf++)
		{
			// Test if the configuration is deletable

			//U1'
			// 0,90,180,270 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,2) && WH(conf,3)
				&& WH(conf,4) && WH(conf,5) && WH(conf,6) && WH(conf,7) && WH(conf,8))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//U2'
			// 0 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,2) && WH(conf,3)
				&& WH(conf,4) && WH(conf,5) && BL(conf,15) && WH_OR(conf, 12, 13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,1) && WH(conf,2) && WH(conf,4) && WH(conf,5)
				&& WH(conf,7) && WH(conf,8) && BL(conf,12) && WH_OR(conf, 10, 15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,3) && WH(conf,4) && WH(conf,5) && WH(conf,6)
				&& WH(conf,7) && WH(conf,8) && BL(conf,10) && WH_OR(conf, 12, 13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,3) && WH(conf,4)
				&& WH(conf,6) && WH(conf,7) && BL(conf,13) && WH_OR(conf, 10, 15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//U3'
			// 0 degrees
			if(WH(conf,1) && WH(conf,2) && WH(conf,4) && WH(conf,5)
				&& WH(conf,10) && WH(conf,13) && BL(conf,12) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,4) && WH(conf,5) && WH(conf,7) && WH(conf,8)
				&& WH(conf,13) && WH(conf,15) && BL(conf,10) && BL(conf,12))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,3) && WH(conf,4) && WH(conf,6) && WH(conf,7)
				&& WH(conf,12) && WH(conf,15) && BL(conf,10) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,3) && WH(conf,4)
				&& WH(conf,10) && WH(conf,12) && BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//U4'
			// 0 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,2) && WH(conf,3)
				&& WH(conf,4) && WH(conf,5) && WH(conf,7) && BL(conf,12)
				&& BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,1) && WH(conf,2) && WH(conf,4) && WH(conf,5)
				&& WH(conf,7) && WH(conf,8) && WH(conf,3) && BL(conf,10)
				&& BL(conf,12) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,3) && WH(conf,4) && WH(conf,5) && WH(conf,6)
				&& WH(conf,7) && WH(conf,8) && WH(conf,1) && BL(conf,10)
				&& BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,3) && WH(conf,4)
				&& WH(conf,6) && WH(conf,7) && WH(conf,5) && BL(conf,10)
				&& BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//U5'
			// 0 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,2) && WH(conf,3)
				&& WH(conf,4) && WH(conf,5) && BL(conf,12) && BL(conf,13)
				&& BL(conf,14) && BL(conf,15) && BL(conf,16))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,1) && WH(conf,2) && WH(conf,4) && WH(conf,5)
				&& WH(conf,7) && WH(conf,8) && BL(conf,9) && BL(conf,10)
				&& BL(conf,12) && BL(conf,14) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,3) && WH(conf,4) && WH(conf,5) && WH(conf,6)
				&& WH(conf,7) && WH(conf,8) && BL(conf,9) && BL(conf,10)
				&& BL(conf,11) && BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,0) && WH(conf,1) && WH(conf,3) && WH(conf,4)
				&& WH(conf,6) && WH(conf,7) && BL(conf,10) && BL(conf,11)
				&& BL(conf,13) && BL(conf,15) && BL(conf,16))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//U6'
			// 0,180 degrees
			if(WH(conf,3) && WH(conf,4) && WH(conf,5) && WH(conf,12)
				&& WH(conf,13) && BL(conf,10) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90,270 degrees
			if(WH(conf,1) && WH(conf,4) && WH(conf,7) && WH(conf,10)
				&& WH(conf,15) && BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D1'
			// 0,90,180,270 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,19) && WH(conf,20)
				&& WH(conf,21) && WH(conf,22) && WH(conf,23) && WH(conf,24)
				&& BL(conf,4))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D2'
			// 0 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,19) && WH(conf,20)
				&& WH(conf,21) && WH_OR(conf,12,13) && BL(conf,4) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,18) && WH(conf,19) && WH(conf,21) && WH(conf,23)
				&& WH(conf,24) && WH_OR(conf,10,15) && BL(conf,4) && BL(conf,12))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,20) && WH(conf,21) && WH(conf,22) && WH(conf,23)
				&& WH(conf,24) && WH_OR(conf,12,13) && BL(conf,4) && BL(conf,10))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,20) && WH(conf,22)
				&& WH(conf,23) && WH_OR(conf,10,15) && BL(conf,4) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D3'
			// 0 degrees
			if(WH(conf,10) && WH(conf,13) && WH(conf,18) && WH(conf,19)
				&& WH(conf,21) && BL(conf,4) && BL(conf,12) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,13) && WH(conf,15) && WH(conf,21) && WH(conf,23)
				&& WH(conf,24) && BL(conf,4) && BL(conf,10) && BL(conf,12))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,12) && WH(conf,15) && WH(conf,20) && WH(conf,22)
				&& WH(conf,23) && BL(conf,4) && BL(conf,10) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,10) && WH(conf,12) && WH(conf,17) && WH(conf,18)
				&& WH(conf,20) && BL(conf,4) && BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D4'
			// 0 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,19) && WH(conf,20)
				&& WH(conf,21) && WH(conf,23) && BL(conf,4) && BL(conf,12)
				&& BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,18) && WH(conf,19) && WH(conf,21) && WH(conf,23)
				&& WH(conf,24) && WH(conf,20) && BL(conf,4) && BL(conf,10)
				&& BL(conf,12) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,20) && WH(conf,21) && WH(conf,22) && WH(conf,23)
				&& WH(conf,24) && WH(conf,18) && BL(conf,4) && BL(conf,10)
				&& BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,20) && WH(conf,22)
				&& WH(conf,23) && WH(conf,21) && BL(conf,4) && BL(conf,10)
				&& BL(conf,13) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D5'
			// 0 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,19) && WH(conf,20)
				&& WH(conf,21) && BL(conf,4) && BL(conf,12)	&& BL(conf,13)
				&& BL(conf,14) && BL(conf,15) && BL(conf,16))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,18) && WH(conf,19) && WH(conf,21) && WH(conf,23)
				&& WH(conf,24) && BL(conf,4) && BL(conf,9)	&& BL(conf,10)
				&& BL(conf,12) && BL(conf,14) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 180 degrees
			if(WH(conf,20) && WH(conf,21) && WH(conf,22) && WH(conf,23)
				&& WH(conf,24) && BL(conf,4) && BL(conf,9)	&& BL(conf,10)
				&& BL(conf,11) && BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 270 degrees
			if(WH(conf,17) && WH(conf,18) && WH(conf,20) && WH(conf,22)
				&& WH(conf,23) && BL(conf,4) && BL(conf,10)	&& BL(conf,11)
				&& BL(conf,13) && BL(conf,15) && BL(conf,16))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}

			//D6'
			// 0 degrees
			if(WH(conf,12) && WH(conf,13) && WH(conf,20) && WH(conf,21)
				&& BL(conf,4) && BL(conf,10) && BL(conf,15))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
			// 90 degrees
			if(WH(conf,10) && WH(conf,15) && WH(conf,18) && WH(conf,23)
				&& BL(conf,4) && BL(conf,12) && BL(conf,13))
			{
				// Pack eight 1bit configurations into one byte
				table[conf] = true;
				continue;
			}
		}
	}

	size_t IndexUD(const Image3d<unsigned char> &img, const Vector3d<size_t> &point)
	{
		Vector3d<int> test;
		size_t conf = 0;
		size_t i = 0;

		for(char y = 1; y > -2; y--)
		{
			for(char z = 1; z > -2; z--)
			{
				for(char x = -1; x < 2; x++)
				{
					if((x | y | z) == 0 || ((x | z) == 0 && y == -1))
						continue;
					
					test = Vector3d<int>(point.x+x, point.y+y, point.z+z);
					if(img.Include(test) && img.GetVoxel(test))
						conf |= 1<<i;

					i++;
				}
			}
		}

		return conf;
	}

	size_t IndexNS(const Image3d<unsigned char> &img, const Vector3d<size_t> &point)
	{
		Vector3d<int> test;
		size_t conf = 0;
		size_t i = 0;

		for(char z = 1; z > -2; z--)
		{
			for(char y = -1; y < 2; y++)
			{
				for(char x = -1; x < 2; x++)
				{
					if((x | y | z) == 0 || ((x | y) == 0 && z == -1))
						continue;

					test = Vector3d<int>(point.x+x, point.y+y, point.z+z);
					if(img.Include(test) && img.GetVoxel(test))
						conf |= 1<<i;

					 i++;
				}
			}
		}

		return conf;
	}

	size_t IndexEW(const Image3d<unsigned char> &img, const Vector3d<size_t> &point)
	{
		Vector3d<int> test;
		size_t conf = 0;
		size_t i = 0;

		for(char x = 1; x > -2; x--)
		{
			for(char z = 1; z > -2; z--)
			{
				for(char y = 1; y > -2; y--)
				{
					if((x | y | z) == 0 || ((y | z) == 0 && x == -1))
						continue;
					
					test = Vector3d<int>(point.x+x, point.y+y, point.z+z);
					if(img.Include(test) && img.GetVoxel(test))
						conf |= 1<<i;

					i++;
				}
			}
		}

		return conf;
	}

	bool IsBorderPoint(const Image3d<uchar> &img, const Vector3d<size_t> &point)
	{
		if(img.OnBorder(point))
			return true; // On the image border
		
		if(!img.GetVoxel(point.x+1, point.y, point.z)||!img.GetVoxel(point.x-1, point.y, point.z)
			||!img.GetVoxel(point.x, point.y+1, point.z)||!img.GetVoxel(point.x, point.y-1, point.z)
			||!img.GetVoxel(point.x, point.y, point.z+1)||!img.GetVoxel(point.x, point.y, point.z-1))
			return true; // Border point
		else
			return false; // Inner point
	}

	bool IsDeletable(const Image3d<unsigned char> &img, Vector3d<size_t> &point, const std::vector<bool> &table)
	{
		Vector3d<int> test;
		uchar l, r;
		size_t config;
		
		// UD test
		test = Vector3d<int>(point.x, point.y+1, point.z);
		l = img.Include(test) && img.GetVoxel(test) == 1;
		test = Vector3d<int>(point.x, point.y-1, point.z);
		r = img.Include(test) && img.GetVoxel(test) == 1;
		if(l + r == 1)
		{
			config = IndexUD(img, point);
			if(table[config])
				return true;
		}

		// NS test
		test = Vector3d<int>(point.x, point.y, point.z+1);
		l = img.Include(test) && img.GetVoxel(test) == 1;
		test = Vector3d<int>(point.x, point.y, point.z-1);
		r = img.Include(test) && img.GetVoxel(test) == 1;
		if(l + r == 1)
		{
			config = IndexNS(img, point);
			if(table[config])
				return true;
		}

		// EW test
		test = Vector3d<int>(point.x+1, point.y, point.z);
		l = img.Include(test) && img.GetVoxel(test) == 1;
		test = Vector3d<int>(point.x-1, point.y, point.z);
		r = img.Include(test) && img.GetVoxel(test) == 1;
		if(l + r == 1)
		{
			config = IndexEW(img, point);
			if(table[config])
				return true;
		}

		return false;
	}

	size_t ThinningIter(std::list<Vector3d<size_t> > &border, Image3d<uchar> &img,
		const std::vector<bool> &table)
	{
		size_t deleted = 0;
		std::list<Vector3d<size_t> > deleteList;

		// Find the points to delete
		for(std::list<Vector3d<size_t> >::iterator i = border.begin(); i != border.end();)
		{
			if(IsDeletable(img, *i, table))
			{
				deleteList.push_back(*i);
				i = border.erase(i);
				deleted++;
			}
			else
			{
				i++;
			}
		}

		// Delete the points
		for(std::list<Vector3d<size_t> >::iterator i = deleteList.begin();
			i != deleteList.end(); i++)
		{
			img.SetVoxel(*i, 0);
			
			// Updatete the border list
			Vector3d<int> point;

			point = Vector3d<int>(i->x+1, i->y, i->z);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
			point = Vector3d<int>(i->x-1, i->y, i->z);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
			point = Vector3d<int>(i->x, i->y+1, i->z);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
			point = Vector3d<int>(i->x, i->y-1, i->z);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
			point = Vector3d<int>(i->x, i->y, i->z+1);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
			point = Vector3d<int>(i->x, i->y, i->z-1);
			if(img.Include(point) && img.GetVoxel(point) == 1)
			{
				border.push_back(point);
				img.SetVoxel(point, 2);
			}
		}

		return deleted;
	}

	void I3D_ALGO_EXPORT Thinning(Image3d<bool> &img)
	{
		std::vector<bool> table;
		Image3d<uchar> tri;
		std::list<Vector3d<size_t> > border;
		size_t deleted;
		uchar *ptr;

		// Create the table
		CreateLUT(table);

		// Convert the image into tricolor representation
		BinaryToScalar(img, tri);

		// Find border voxels
		ptr = tri.GetFirstVoxelAddr();
		for(size_t i = 0; i < tri.GetImageSize(); i++, ptr++)
		{
			if(*ptr == 1 && IsBorderPoint(tri, tri.GetPos(i)))
			{
				border.push_back(tri.GetPos(i));
				*ptr = 2;
			}
		}

		size_t iters = 0;
		// Thinning
		do
		{
			deleted = ThinningIter(border, tri, table);
			iters++;
		}
		while(deleted > 0);

		// Save the result
		i3d::GrayToBinary(tri, img);
		for(std::list<Vector3d<size_t> >::iterator i = border.begin(); i != border.end(); i++)
		{
			img.SetVoxel(*i, true);
		}
	}
}  // namespace i3d
